@@ -1,8 +1,3 @@
-"""
-Set up backoff module for wikibaseintegrator
-
-"""
-
 import sys
 from functools import partial
 
@@ -23,9 +18,7 @@ def backoff_hdlr(details):
     exc_type, exc_value, _ = sys.exc_info()
     if exc_type == JSONDecodeError:
         print(exc_value.doc)
-    print("Backing off {wait:0.1f} seconds afters {tries} tries "
-          "calling function with args {args} and kwargs "
-          "{kwargs}".format(**details))
+    print("Backing off {wait:0.1f} seconds afters {tries} tries calling function with args {args} and kwargs {kwargs}".format(**details))
 
 
 def check_json_decode_error(e):
@@ -38,9 +31,7 @@ def check_json_decode_error(e):
     return type(e) == JSONDecodeError and str(e) != "Expecting value: line 1 column 1 (char 0)"
 
 
-exceptions = (requests.exceptions.Timeout, requests.exceptions.ConnectionError,
-              requests.HTTPError, JSONDecodeError)
+exceptions = (requests.exceptions.Timeout, requests.exceptions.ConnectionError, requests.HTTPError, JSONDecodeError)
 
-wbi_backoff = partial(backoff.on_exception, backoff.expo, exceptions, max_value=get_config("BACKOFF_MAX_VALUE"),
-                      giveup=check_json_decode_error, on_backoff=backoff_hdlr, jitter=None,
-                      max_tries=get_config("BACKOFF_MAX_TRIES"))
+wbi_backoff = partial(backoff.on_exception, backoff.expo, exceptions, max_value=get_config("BACKOFF_MAX_VALUE"), giveup=check_json_decode_error,
+                      on_backoff=backoff_hdlr, jitter=None, max_tries=get_config("BACKOFF_MAX_TRIES"))
